#!/usr/bin/env python
# -*- coding: iso-8859-15 -*-

# GIMP script to render a wave along a path
# (c) Ofnuts 2011
#
#   History:
#
#   v0.0: 2011-06-29:  first published version
#   v0.1: 2011-06-29:  refactor, add circles and ellipses
#   v0.2: 2011-07-19:  improve parameter dialogs
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import math, random, os, sys, time
from gimpfu import *

# ----------------------------------------------------------------
# Point with additional information: tangent, direction
# and some methods to build more points relative to it
# ----------------------------------------------------------------
class Point:
	x=0
	y=0
	theta=0 # oriented angle of our tangent
	
	def __init__(self,x,y,theta):
		self.x=x
		self.y=y
		self.theta=theta
		
	def __str__(self):
		return 'Point<(%6.4f,%6.4f),%5.3f>' % (self.x,self.y,(self.theta*180)/math.pi)

	def __repr__(self):
		return self.__str__()

	def coords(self):
		return [self.x,self.y]


	# Computes point at signed distance and given oriented angle
	def pointAtAngleDistance(self,distance,alpha):
		deltaX=distance*math.cos(alpha)
		deltaY=distance*math.sin(alpha)
		x=self.x+deltaX
		y=self.y+deltaY
		return Point(x,y,self.theta)


	# Computes the point at signed distance D from us, so that the vector is normal to our tangent
	# Side=+1: right, Side=-1: left
	# We support both a side specification and a signed distance, 
	# the effects are similar but the purposes are different
	def pointOnPerpendicular(self,distance,side):
		alpha=self.theta+(math.pi*side/2)
		p=self.pointAtAngleDistance(distance,alpha)
		return p

	# Computes the point at signed distance D from us, along our tangent
	# Side=+1: forward, Side=-1: backward
	# We support both a side specification and a signed distance, 
	# the effects are similar but the purposes are different positions
	def pointOnTangent(self,distance,side):
		alpha=self.theta
		if side == -1:
			alpha=alpha+math.pi
		p=self.pointAtAngleDistance(distance,alpha)
		return p

	# Computes the point on diagonal at signed distances dT and dP from us
	# SideP=+1: right, SideP=-1: left
	# SideT=+1: forward, SideT=-1: backward
	# We support both a side specification and a signed distance, 
	def pointOnDiagonal(self,dT,dP,sideT,sideP):
		# deltas from tangent
		alphaT=self.theta
		if sideT == -1:
			alphaT=alphaT+math.pi
		deltaTX=dT*math.cos(alphaT)
		deltaTY=dT*math.sin(alphaT)

		# deltas from perpendicular
		alphaP=self.theta+(math.pi*sideP/2)
		deltaPX=dP*math.cos(alphaP)
		deltaPY=dP*math.sin(alphaP)

		x=self.x+deltaTX+deltaPX
		y=self.y+deltaTY+deltaPY
		return Point(x,y,self.theta)

# ----------------------------------------------------------------
# Generic class to hold a triplet of points:
# control points and backward/forward handles
# ----------------------------------------------------------------
class Triplet(object):
	bh=None
	fh=None
	cp=None
	
	def __init__(self,bh,cp,fh):
		self.bh=bh
		self.cp=cp
		self.fh=fh
	
	def points(self):
		p=[]
		p.extend(self.bh.coords());
		p.extend(self.cp.coords());
		p.extend(self.fh.coords());
		return p

	def __repr__(self):
		return self.__str__()

	def __str__(self):
		return 'Triplet<%s,%s,%s>' % (self.bh,self.cp,self.fh)

# ----------------------------------------------------------------
# Generic class to compute control points for curvy waves defined
# by extrema and zeros: sine, ellipse, circle...
# ----------------------------------------------------------------
class CurvyControlPointsRenderer(object):
	def max(self,p,scaleXb,scaleXf,scaleY,side):
		pass

	def zero(self,p,scaleXb,scaleXf,scaleYb,scaleYf,direction):
		pass


# -----------------------------------------------------------------------------
# Class for the approximation of a sine arc (a quarter of a period), as a
# Bezier curve where the anchors are an extremum and the intersection of the axis
#	
# The constants used here have no mathematical justification. They have been
# obtained as follows:
# 	
# - have some utility draw a quarter of sine curve, from (0,0) to (pi/2,1) 
# - import image in Gimp, scale if needed: 1000px to 1571px
# - create a path with two anchors, one on (0,0), one on (pi/2,1)
# - add handles with following constraints:
# 	- handle on (pi/2,1) has Y=1 (horizontal tangent)
# 	- handle for (0,0) has X=Y (dsinx(x)/dy for x=0 is cos(0)=1)
# - try to get best match on curve graphically
# - extract values from the handle coordinates
# - feed to python code that will run for a while stepping both values
# in a wide range around these two values to find those that produce a
# minimal global error (sum of abs(evaluated_Y-true_Y) over 100 samples).
# 
# -----------------------------------------------------------------------------

class SineControlPointsRenderer(CurvyControlPointsRenderer):
	# Constant for handles on extrema, rescaled for 1:1 curve (computed on 1:pi/2)
	KS=0.56860*2/math.pi
	# Constants for handles on intersections with X axis
	KIT=0.51180*2/math.pi
	KIP=0.51180

	def max(self,p,scaleXb,scaleXf,scaleY,side):
		cp=p.pointOnPerpendicular(scaleY,side)
		fh=cp.pointOnTangent(scaleXf*self.KS,1)
		bh=cp.pointOnTangent(scaleXb*self.KS,-1)
		return Triplet(bh,cp,fh)
		
	def zero(self,p,scaleXb,scaleXf,scaleYb,scaleYf,direction):
		cp=p
		fh=cp.pointOnDiagonal(scaleXf*self.KIT,scaleYf*self.KIP,1,direction)
		bh=cp.pointOnDiagonal(scaleXb*self.KIT,scaleYb*self.KIP,-1,-direction)
		return Triplet(bh,cp,fh)

# -----------------------------------------------------------------------------
# Classes for the approximation of a circle/ellipse arc (a quarter of a period), as a
# Bezier curve where the anchors are an extremum and the intersection of the axis
#	
# They use the well known "kappa" constant: (4/3)*(sqrt(2)-1)
# -----------------------------------------------------------------------------

KAPPA=(4./3.)*(math.sqrt(2)-1)

class EllipseControlPointsRenderer(CurvyControlPointsRenderer):

	def max(self,p,scaleXb,scaleXf,scaleY,side):
		cp=p.pointOnPerpendicular(scaleY,side)
		fh=cp.pointOnTangent(scaleXf*KAPPA,1)
		bh=cp.pointOnTangent(scaleXb*KAPPA,-1)
		return Triplet(bh,cp,fh)
		
	def zero(self,p,scaleXb,scaleXf,scaleYb,scaleYf,direction):
		cp=p
		fh=cp.pointOnPerpendicular(scaleYf*KAPPA,direction)
		bh=cp.pointOnPerpendicular(scaleYb*KAPPA,-direction)
		return Triplet(bh,cp,fh)

# This class enforces the circular shape, so the Y scale is ignored
class CircleControlPointsRenderer(CurvyControlPointsRenderer):

	def max(self,p,scaleXb,scaleXf,scaleY,side):
		cp=p.pointOnPerpendicular(scaleXb,side)
		fh=cp.pointOnTangent(scaleXf*KAPPA,1)
		bh=cp.pointOnTangent(scaleXb*KAPPA,-1)
		return Triplet(bh,cp,fh)
		
	def zero(self,p,scaleXb,scaleXf,scaleYb,scaleYf,direction):
		cp=p
		fh=cp.pointOnPerpendicular(scaleXf*KAPPA,direction)
		bh=cp.pointOnPerpendicular(scaleXb*KAPPA,-direction)
		return Triplet(bh,cp,fh)

#----------------------------------------------------
# Class to add some information to a regular Stroke
#
# Experience shows that there are cumulative errors when the Gimp code
# computes the length of the individual Bezier curves. To maintain a 
# sufficient total precision, we have to be more stringent when the
# stroke has more points
#----------------------------------------------------
class DirectionStroke:
	
	stroke=None
	length=0
	firstPoint=None
	previousPoint=None
	precision=.1
		
	def __init__(self,stroke,precision):
		self.stroke=stroke
		points,closed=self.stroke.points
		bezierCurves=(len(points)/3)-1
		self.precision=precision/bezierCurves
		#self.precision=precision
		# The stroke length, computed once for all. It appears that due to precision errors;
		# self.stroke.get_point_at_dist(self.length) may not return a valid point.
		self.length=stroke.get_length(self.precision)
		self.previousPoint=None

	# Some utility functions to handle points
	def getPoint(self,n):
		points=self.stroke.points[0]
		start=(n*2)%len(points)
		return points[start:start+2]
		
	def isClosed(self):
		return self.stroke.points[1]
		
	def isSamePoint(self,p1,p2):
		return math.fabs(p1[0]-p2[0])< self.precision and math.fabs(p1[1]-p2[1])< self.precision

	# Two variants to obtain the oriented angle of the tangent at point
	# In the first one, we know the real slope returned from the get_point_at_dist() Gimp API
	# In the second we are left to our own devices, and compute an approximate slope
	def computeThetaWithSlope(self,fromPoint,toPoint,slope):
		dX=toPoint[0]-fromPoint[0]
		dY=toPoint[1]-fromPoint[1]
		
		if math.fabs(dX) < .001:
			theta=math.copysign(math.pi/2,dY)
		else:
			theta=math.atan(slope)
			if dX<0:
				theta=theta+math.pi
		return theta % (math.pi*2)
		
	def computeTheta(self,fromPoint,toPoint):
		dX=toPoint[0]-fromPoint[0]
		dY=toPoint[1]-fromPoint[1]
		
		if math.fabs(dX) < .001:
			theta=math.copysign(math.pi/2,dY)
		else:
			theta=math.atan(dY/dX)
			if dX<0:
				theta=theta+math.pi
		return theta % (math.pi*2)
	
	# Facade method, returns only if point is OK
	def getRawPointAtDist(self,dist):
		x,y,slope,valid=self.stroke.get_point_at_dist(dist,self.precision)
		if not valid: # Normally quite unlikely, since we avoid the cases where this could happen
			raise Exception('No point found at %6.4f in stroke %s with length %6.4f' % (dist,self.stroke.ID,self.length))
		return x,y,slope
	
	# Enhanced version of stroke.get_point_at_dist(...)
	# That returns information about the path direction
	# Getting the last point is a special case to avoid precision problems
	def getPointAtDist(self,dist):
		# get rid of the special case (all done in othe rmethod)
		if  dist<0:
			return self.getLastPoint()

		# Now process regular case
		x,y,slope=self.getRawPointAtDist(dist)
		
		# Find the direction of the curve at given point, by comparing coordinates with nearby point
		# further along the path at an arbitrary distance delta.
		# We assume that dist+delta is still on the curve (ie, dist no too close to end)
		#delta=self.precision*20
		delta=.01
		toX,toY,s=self.getRawPointAtDist(dist+delta) 
		theta=self.computeThetaWithSlope([x,y],[toX,toY],slope)

		return Point(x,y,theta)
	
	# Return our last point directly (this avoid cases where 
	# dist asked is slighly greater than computed length)
	# Handling of open and closed stroke is different, since 
	# in closed strokes, the last point is actually the first.
	def getLastPoint(self):
		# if closed last point of stroke is same as first
		points,closed=self.stroke.points
		
		# The slope on the last point is the vector from the previous control point 
		# that isn't identical to the last point
		# So we have to try, in that order
		# - the backward control point of the last point
		# - the forward control point of the next-to-last point
		# - the next-to-last point
		# and since it happens that in closed strokes (frequent in fonts) the last point  
		# in the list can be identical to the first (double closure) we have to go down
		# one more level.
		# In other words for an open stroke containing N points (and N/3 triplets)
		# In closed stroke
		# - last point is [1] or 1%N
		# - vector is determined by [0][N-1][N-2][N-3],[N-4],[N-5](or [i%N for i in range(0,-6,-1)])
		# In open stroke:
		# - last point is [N-2] ([N-1] is its forward handle) or -2%N
		# - vector is determined by [N-3],[N-4],[N-5] (or [i%N for i in range(-3,-6,-1)])
		
		theta=None
		thetaDone=False
		if closed:
			lastPoint=self.getPoint(1)
			checkedPoints=range(0,-6,-1)
		else:
			lastPoint=self.getPoint(-2)
			checkedPoints=range(-3,-6,-1)
			
		for i in checkedPoints:
			tangentOriginPoint=self.getPoint(i)
			if self.isSamePoint(lastPoint,tangentOriginPoint):
				continue # can't compute slope here
			theta=self.computeTheta(tangentOriginPoint,lastPoint)
			thetaDone=True
			break;
		
		# At that point we should have found theta
		if not thetaDone:
			raise Exception('Could not compute tangent theta on last point (at (%6.4f,%6.4f)) of stroke %s (too many identical points)' % (lastPoint[0],lastPoint[1],self.stroke.ID))

		return Point(lastPoint[0],lastPoint[1],theta)


LAYERS_NAMES=['A','B']

# -----------------------------------------------------------------------------
# Base class for renderers
#
# Iterate the strokes in the path, and within the strokes, iterates the periods
# Also holds various house-keeping methods for paths and strokes and some
# permanent inter-stroke variables
# -----------------------------------------------------------------------------
class WaveRenderer(object):
	image=None
	drawable=None
	waveLength=0
	actualWaveLength=0
	amplitude=0
	asymmetry=50
	side=0
	split=False
	close=True
	
	parameters=''

	precision=.1
	totalLength=0
	pathSoFar=0
	
	refVectors=None
	vectors=[None,None]
	refStroke=None

	#
	# Variables actually used by our derived classes
	#
	# Current strokes in the making
	points=[None,None]
	# First point generated, stored to be resued on last segment
	firstPoint=None

	def  __init__(self,parameters,image,path,waveLength,amplitude,side,asymmetry,split):
		self.image=image
		self.refVectors=path
		self.waveLength=waveLength
		self.amplitude=amplitude
		self.side=side
		self.asymmetry=(asymmetry+50)/100.
		self.split=split
		self.parameters=parameters
		
	# Add a triplet of points, from a SineMax or an SineZero
	def addTriplet(self,p,layer=0):
		points=self.points[layer]
		points.extend(p.points())

	# Add a point without tangents
	def addHardPoint(self,p,layer=0):
		points=self.points[layer]
		points.extend(p.coords())
		points.extend(p.coords())
		points.extend(p.coords())
		
	# Add the current list of points as a stroke to the path
	def finishStroke(self,close,layer=0):
		#print 'Creating new stroke'
		path=self.getPath(layer)
		points=self.points[layer]
		sid = pdb.gimp_vectors_stroke_new_from_points(path, VECTORS_STROKE_TYPE_BEZIER, len(points), points, close)
		self.points[layer]=[]

	# Where the action is
	def runStroke(self):
		# ensure we get new strokes created for this stroke
		timeBegin=time.time()
		self.currentSide=-1
		self.currentStroke=None

		# Compute the actual waveLength, to make sure we have an integer number of periods
		self.actualWaveLength=self.waveLength
		periods=int(round(self.refStroke.length/self.waveLength))
		self.actualWaveLength=self.refStroke.length/periods
		print 'Stroke: %4.3f requested waveLength: %4.3f, adjusted to %4.3f for %d periods' % (self.refStroke.length,self.waveLength, self.actualWaveLength, periods)
		starting=self.refStroke.getPointAtDist(0)
		ending=self.refStroke.getLastPoint()
		print 'Starting: %s, ending: %s' % (starting,ending)
		
		self.startOfStroke()
		for period in range(periods):
			pdb.gimp_progress_update((1.+period)/periods)
			self.processPeriod(period,period==(periods-1))		
		self.endOfStroke()
		timeEnd=time.time()
		print 'Stroke process time: %5.3f' % (timeEnd-timeBegin)
		
	def getPathName(self,layer=0):
		name= '%s: %s' % (self.refVectors.name,self.parameters)
		if self.split:
			name=name+'-'+LAYERS_NAMES[layer]
		return name
		
	def getPath(self,layer=0):
		#print 'Getting path'
		if not self.vectors[layer]:
 			self.vectors[layer]=pdb.gimp_vectors_new(self.image, self.getPathName(layer))
			pdb.gimp_image_add_vectors(self.image, self.vectors[layer], 0)

		return self.vectors[layer]
	
		
	# Defined for the benefit of our subclasses
	def startOfPath(self):
		pass
	def endOfPath(self):
		pass
	def startOfStroke(self):
		print 'Processing start of stroke in generic'
		pass
	def endOfStroke(self):
		print 'Processing end of stroke in generic'
		pass
	
	# Check startup conditions, and compute total path length
	def okToRun(self):
		if self.refVectors==None:
			raise Exception('No active path in image')
		if not self.refVectors :
			raise Exception('No elements in active path')
			
		for stroke in self.refVectors.strokes:
			length=stroke.get_length(self.precision)
			self.totalLength=self.totalLength+length
		if self.totalLength<=0:
			raise Exception('Active path of length 0')

	def runPath(self):
		print '######################################################################'
		self.pathSoFar=0.
		# Run on the strokes
		for i in range(len(self.refVectors.strokes)):
			stroke=self.refVectors.strokes[i]
			pdb.gimp_progress_update(0.)
			pdb.gimp_progress_set_text('Stroke %d of %d' % (i+1,len(self.refVectors.strokes)))
			print 'Running for stroke %d' % stroke.ID
			self.refStroke=DirectionStroke(stroke,self.precision)
			self.runStroke()
			self.refStroke=None
		self.endOfPath()
		#Make the vectors visible
		for vector in self.vectors:
			if vector:
				vector.visible=True

	# Run the whole process
	def run(self):
		try:
			self.okToRun()
			self.runPath()
		except Exception as e:	
			print e
			pdb.gimp_message(e.args[0])
		pdb.gimp_displays_flush()
		return;

# -----------------------------------------------------------------------------
# Triangle wave renderers
#
# In split mode the slayer strokes corresponds to the 
# segments between the extrema
# -----------------------------------------------------------------------------
# Cosine phase triangle wave renderer
# 
# Periods:
# 	Single mode:
# 		- Add the the first two extrema to layer A
# 	Split mode:
# 		- Finish pending segment on layer A with first extremum (except on first period)
# 		- Add segment between two extrema on layer B
# 		- Start new segment on layer A from second extremum
# 
# Start of stroke:
# 	- Nothing to do
# 
# End of stroke:
# 	Open stroke:
# 		The process is conceptually different, but ends up being
# 		the same in both modes:
# 		
# 		Single mode:
# 			- Add last point on stroke (image of n*2*pi,1) to 
# 			list of points in layer A and finish stroke
# 		Split mode:
# 			- Add last point on stroke (image of n*2*pi,1) to
# 			segment on layer A and finish stroke
# 
# 	Closed stroke:
# 		Continous mode:
# 			- Nothing to do, just mark the result stroke as closed
# 			so this will generate the segment from the 2nd extremum
# 			of the last period to the first extremum of the first period.
# 		Split mode:
# 			- Finish pending segment on layer A with first extremum
# 			of first period.
# 
class TriangleWaveRenderer_Cos(WaveRenderer):
	
	def  __init__(self,parameters,image,path,waveLength,amplitude,side,asymmetry,split):		
		super(TriangleWaveRenderer_Cos, self).__init__(parameters,image,path,waveLength,amplitude,side,asymmetry,split)
		
	def startOfStroke(self):
		self.points=[[],[]]

	def processPeriod(self,period,last):
		p0=self.refStroke.getPointAtDist(period*self.actualWaveLength)
		p1=self.refStroke.getPointAtDist((period+self.asymmetry)*self.actualWaveLength)
		c0=p0.pointOnPerpendicular(self.amplitude,self.side)
		c1=p1.pointOnPerpendicular(self.amplitude,-self.side)

		if self.split:
			# finish previous segment on layer A unless on first period)
			if period !=0:
				self.addHardPoint(c0,0)
				self.finishStroke(False,0)
			
			# create segment on layer B
			self.addHardPoint(c0,1)
			self.addHardPoint(c1,1)
			self.finishStroke(False,1)

			# start new segment for layer A
			# to be finished in next period
			self.addHardPoint(c1,0)
			
			# If on first period, store first point
			if period==0:
				self.firstPoint=c0
		else:
			self.addHardPoint(c0)
			self.addHardPoint(c1)
		
	def endOfStroke(self):
		closed=self.refStroke.isClosed()
		if not closed:
			p0=self.refStroke.getLastPoint()
			c0=p0.pointOnPerpendicular(self.amplitude,self.side)
			self.addHardPoint(c0,0)
			self.finishStroke(False,0)
		else:
			if self.split:
				self.addHardPoint(self.firstPoint,0)
				print self.points[0]
				self.finishStroke(False,0)
				
			else:
				self.finishStroke(True,0)


# -----------------------------------------------------------------------------
# Sine phase triangle wave renderer
# 
# Periods:
# 	Single mode:
# 		- Add the two points corresponding to the two extrema
# 	Split mode:
# 		- Finish pending segment on layer A with first extremum
# 		- Add segment between two extrema on layer B
# 		- Start new segment on layer A from second extremum
# 
# Start of stroke:
# 	Open stroke:
# 		The process is conceptually different, but ends up being
# 		the same in both modes:
# 		
# 		Single mode:
# 			-  Add first point on stroke (image of 0,0) to layer A
# 			as start of half segment at begining of stroke
# 		Split mode:
# 			-  Add first point on stroke (image of 0,0) to layer A
# 			as start of half segment for layer A
# 	Closed stroke:
# 		- Nothing to do, closure handled in end of stroke
# 
# End of stroke:
# 	Open stroke:
# 		The process is conceptually different, but ends up being
# 		the same in both modes:
# 		
# 		Single mode:
# 			- Add last point on stroke (image of n*2*pi,0) to 
# 			list of points in layer A and finish stroke
# 		Split mode:
# 			- Add last point on stroke (image of n*2*pi,0) to
# 			segment on layer A and finish stroke
# 
# 	Closed stroke:
# 		Continous mode:
# 			- Nothing to do, just mark the result stroke as closed
# 			so this will generate the segment from the 2nd extremum
# 			of the last period to the first extremum of the first period.
# 		Split mode:
# 			- Finish pending segment on layer A with first extremum
# 			of first period.
# 
class TriangleWaveRenderer_Sin(WaveRenderer):
	
	def  __init__(self,parameters,image,path,waveLength,amplitude,side,asymmetry,split):		
		super(TriangleWaveRenderer_Sin, self).__init__(parameters,image,path,waveLength,amplitude,side,asymmetry,split)
		
	def startOfStroke(self):
		self.points=[[],[]]
		self.firstPoint=None
		if not self.refStroke.isClosed():
			self.addHardPoint(self.refStroke.getPointAtDist(0),0)

	def processPeriod(self,period,last):
		p0=self.refStroke.getPointAtDist((period+self.asymmetry/2)*self.actualWaveLength)
		p1=self.refStroke.getPointAtDist((period+1-self.asymmetry/2)*self.actualWaveLength)
		c0=p0.pointOnPerpendicular(self.amplitude,self.side)
		c1=p1.pointOnPerpendicular(self.amplitude,-self.side)
		if self.split:
			# finish previous segment on layer A
			self.addHardPoint(c0,0)
			self.finishStroke(False,0)
			
			# create segment on layer B
			self.addHardPoint(c0,1)
			self.addHardPoint(c1,1)
			self.finishStroke(False,1)

			# start new segment for layer A
			# to be finished in next period
			self.addHardPoint(c1,0)
			
			# If on first period, store first point
			if period==0:
				self.firstPoint=c0
		else:
			self.addHardPoint(c0)
			self.addHardPoint(c1)

	def endOfStroke(self):
		closed=self.refStroke.isClosed()
		if not closed:
			self.addHardPoint(self.refStroke.getLastPoint(),0)
			self.finishStroke(False,0)
		else:
			if self.split:
				self.addHardPoint(self.firstPoint,0)
				self.finishStroke(False,0)
			else:
				self.finishStroke(True,0)

# -----------------------------------------------------------------------------
# Square wave renderers
#
# In split mode the layer strokes correspond to a "parallel" segment followed by a 
# "perpendicular" one: "top" and "top-to-bottom" on A, and "bottom" and
# "bottom-to-top" on B.
# -----------------------------------------------------------------------------
# Cosine phase square wave renderers
# 
# Periods:
# 	Single mode:
# 		- Add the second point to the top segment
# 		- Add the two points of the bottom segment
# 		- Add the first point of the next top segment
# 	Split mode:
# 		- Finish pending top segment on layer A stroke 
# 		- Add top-to-bottom segment to layer A stroke and finish it.
# 		- Start layer B stroke with bottoml segment
# 		- Finish layer B stroke with bottom-to-top segment
# 		- Start layer A stroke with first point
# 
# Start of stroke:
# 	Open stroke:
# 		The process is conceptually different, but ends up being
# 		the same in both modes:
# 		
# 		Single mode:
# 			-  Add first point on stroke (image of 0,0) to layer A
# 			as start of half top segment at begining of stroke
# 		Split mode:
# 			-  Add first point on stroke (image of 0,0) to layer A
# 			as start of half top segment for layer A
# 	Closed stroke:
# 		- Nothing to do, closure handled in end of stroke
# 
# End of stroke:
# 	Open stroke:
# 		The process is conceptually different, but ends up being
# 		the same in both modes:
# 		
# 		Single mode:
# 			- Add last point on stroke (image of n*2*pi,1) to list of
# 			of points in layer A and finish stroke with half top segment
# 		Split mode:
# 			- Add last point on stroke (image of n*2*pi,1) to
# 			segment on layer A and finish stroke with half top segment
# 
# 	Closed stroke:
# 		Continous mode:
# 			- Nothing to do, just mark the result stroke as closed
# 			so this will generate the top segment 
# 			of the last period to the first extremum of the first period.
# 		Split mode:
# 			- Finish pending top segment on layer A with first extremum
# 			of first period (i.e. firstPoint)
# 
class SquareWaveRenderer_Cos(WaveRenderer):
	
	def  __init__(self,parameters,image,path,waveLength,amplitude,side,asymmetry,split):		
		super(SquareWaveRenderer_Cos, self).__init__(parameters,image,path,waveLength,amplitude,side,asymmetry,split)
		
	def startOfStroke(self):
		self.points=[[],[]]
		self.firstPoint=None
		if not self.refStroke.isClosed():
			p0=self.refStroke.getPointAtDist(0)
			c0=p0.pointOnPerpendicular(self.amplitude,self.side)
			self.addHardPoint(c0,0)

	def processPeriod(self,period,last):
		p0=self.refStroke.getPointAtDist((period+(self.asymmetry/2))*self.actualWaveLength)
		p1=self.refStroke.getPointAtDist((period+1-(self.asymmetry/2))*self.actualWaveLength)
		c0=p0.pointOnPerpendicular(self.amplitude,self.side)
		c1=p0.pointOnPerpendicular(self.amplitude,-self.side)
		c2=p1.pointOnPerpendicular(self.amplitude,-self.side)
		c3=p1.pointOnPerpendicular(self.amplitude,self.side)

		if self.split:
			# finish half-wave stroke on layer A 
			self.addHardPoint(c0,0)
			self.addHardPoint(c1,0)
			self.finishStroke(False,0)
			
			# produce half-wave stroke on layer B
			self.addHardPoint(c1,1)
			self.addHardPoint(c2,1)
			self.addHardPoint(c3,1)
			self.finishStroke(False,1)

			# start new  half-wave stroke on layer A
			# to be finished in next period
			self.addHardPoint(c3,0)
			
			# If on first period, store first point
			if period==0:
				self.firstPoint=c0
		else:
			self.addHardPoint(c0)
			self.addHardPoint(c1)
			self.addHardPoint(c2)
			self.addHardPoint(c3)
		
	def endOfStroke(self):
		closed=self.refStroke.isClosed()
		if not closed:
			p0=self.refStroke.getLastPoint()
			c0=p0.pointOnPerpendicular(self.amplitude,self.side)
			self.addHardPoint(c0,0)
			self.finishStroke(False,0)
		else:
			if self.split:
				self.addHardPoint(self.firstPoint,0)
				self.finishStroke(False,0)
			else:
				self.finishStroke(True,0)

# -----------------------------------------------------------------------------
# Sine phase square wave renderers
# 
# Periods:
# 	Single mode:
# 		- Add top segment
# 		- Add top-to-bottom segment
# 		- Add bottom segment
# 	Split mode:
# 		- Add bottom-to-top segment on layer A stroke and finish it
# 		- Add top segment to layer B stroke 
# 		- Add top-to-bottom segment to layer B stroke and finish it.
# 		- Add bottom segment to layer A stroke
# 
# Start of stroke:
# 	Open stroke:
# 		The process is conceptually different, but ends up being
# 		the same in both modes:
# 		
# 		Single mode:
# 			-  Add first point on stroke (image of 0,0) on layer A
# 		Split mode:
# 			-  Add segment for image(0,0) to image of (0,1) to 
# 			layer A (half botto-to-top segment stroke on A 
# 			will be created by first period)
# 	Closed stroke:
# 		- Nothing to do, closure handled in end of stroke
# 
# End of stroke:
# 	Open stroke:
# 		The process is conceptually different, but ends up being
# 		the same in both modes:
# 		
# 		Single mode:
# 			- Add last point on stroke (image of n*2*pi,0) to list of
# 			of points in layer A and finish stroke with half 
# 			bottom-to-top segment
# 		Split mode:
# 			- Add last point on stroke (image of n*2*pi,0) to
# 			segment on layer A and finish stroke with half 
# 			bottom-to-top segment
# 	Closed stroke:
# 		Single mode:
# 			- Nothing to do, just mark the result stroke as closed so 
# 			this will generate the final bottom-to-top segment from the 
# 			bottom of the last period to the first point of the first period.
# 		Split mode:
# 			- Finish pending layer A stroke with image of (n*2*pi,1)
# 			of first period (i.e. firstPoint)
# 
class SquareWaveRenderer_Sin(WaveRenderer):
	
	def  __init__(self,parameters,image,path,waveLength,amplitude,side,asymmetry,split):		
		super(SquareWaveRenderer_Sin, self).__init__(parameters,image,path,waveLength,amplitude,side,asymmetry,split)
		
	def startOfStroke(self):
		self.points=[[],[]]
		self.firstPoint=None
		if not self.refStroke.isClosed():
			self.addHardPoint(self.refStroke.getPointAtDist(0),0)

	def processPeriod(self,period,last):
		p0=self.refStroke.getPointAtDist(period*self.actualWaveLength)
		p1=self.refStroke.getPointAtDist((period+self.asymmetry)*self.actualWaveLength)
		if last:
			p2=self.refStroke.getLastPoint()
		else:
			p2=self.refStroke.getPointAtDist((period+1)*self.actualWaveLength)
		c0=p0.pointOnPerpendicular(self.amplitude,self.side)
		c1=p1.pointOnPerpendicular(self.amplitude,self.side)
		c2=p1.pointOnPerpendicular(self.amplitude,-self.side)
		c3=p2.pointOnPerpendicular(self.amplitude,-self.side)

		if self.split:
			# finish previous half-wave on layer A
			self.addHardPoint(c0,0)
			self.finishStroke(False,0)
			
			# create half-wave on layer B
			self.addHardPoint(c0,1)
			self.addHardPoint(c1,1)
			self.addHardPoint(c2,1)
			self.finishStroke(False,1)

			# start half-wave for layer B
			# to be finished in next period
			self.addHardPoint(c2,0)
			self.addHardPoint(c3,0)
			
			# If on first period, store first point
			if period==0:
				self.firstPoint=c0
		else:
			self.addHardPoint(c0)
			self.addHardPoint(c1)
			self.addHardPoint(c2)
			self.addHardPoint(c3)

	def endOfStroke(self):
		closed=self.refStroke.isClosed()
		if not closed:
			self.addHardPoint(self.refStroke.getLastPoint(),0)
			self.finishStroke(False,0)
		else:
			if self.split:
				self.addHardPoint(self.firstPoint,0)
				self.finishStroke(False,0)
			else:
				self.finishStroke(True,0)

# -----------------------------------------------------------------------------
# Wave renderers for symmetrical zeros
#
# In split mode the layer strokes correspond to the sine arcs between two extrema:
# "top-to-bottom" on A, and "bottom-to-top" on B.
# -----------------------------------------------------------------------------
# Cosine phase sine wave renderer
#
# Periods:
# 	Single mode:
# 		- generate full swing except last point: extremum, zero, extremum,
# 		zero.
# 	Split mode:
# 		- Finish pending segment on layer A with first extremum (except on first period)
# 		- Add segments between two extrema on layer B: extremum, zero, extremum
# 		- Start new segment on layer A: extremum, zero
# 
# Start of stroke:
# 	- Nothing to do
# 
# End of stroke:
# 	Open stroke:
# 		The process is conceptually different, but ends up being
# 		the same in both modes:
# 		
# 		Single mode:
# 			- Add last point on stroke (image of n*2*pi,1) to 
# 			list of points in layer A and finish stroke
# 		Split mode:
# 			- Add last point on stroke (image of n*2*pi,1) to
# 			segment on layer A and finish stroke
# 
# 	Closed stroke:
# 		Continous mode:
# 			- Nothing to do, just mark the result stroke as closed
# 			so this will generate the segment from the 2nd extremum
# 			of the last period to the first extremum of the first period.
# 		Split mode:
# 			- Finish pending segment on layer A with first extremum
# 			of first period (firstPoint)
# 
class SymmetricZeroWaveRenderer_Cos(WaveRenderer):
	
	# Curvy Control Points renderer
	ccpr=None
	
	def  __init__(self,ccpr,parameters,image,path,waveLength,amplitude,side,asymmetry,split):		
		self.ccpr=ccpr
		super(SymmetricZeroWaveRenderer_Cos, self).__init__(parameters,image,path,waveLength,amplitude,side,asymmetry,split)
		
	def startOfStroke(self):
		self.points=[[],[]]
		self.firstPoint=None

	def processPeriod(self,period,last):
		arcLen0=self.actualWaveLength*self.asymmetry/2
		arcLen1=self.actualWaveLength*(1-self.asymmetry)/2
		p0dist=period*self.actualWaveLength
		p1dist=p0dist+arcLen0
		p2dist=p1dist+arcLen0
		p3dist=p2dist+arcLen1
		p0=self.refStroke.getPointAtDist(p0dist)
		p1=self.refStroke.getPointAtDist(p1dist)
		p2=self.refStroke.getPointAtDist(p2dist)
		p3=self.refStroke.getPointAtDist(p3dist)
		
		c0=self.ccpr.max(p0,arcLen1,arcLen0,self.amplitude,self.side)
		c1=self.ccpr.zero(p1,arcLen0,arcLen0,self.amplitude,self.amplitude,-self.side)
		c2=self.ccpr.max(p2,arcLen0,arcLen1,self.amplitude,-self.side)
		c3=self.ccpr.zero(p3,arcLen1,arcLen1,self.amplitude,self.amplitude,self.side)

		if self.split:
			if period:
				# finish half-wave stroke on layer A 
				self.addTriplet(c0,0)
				self.finishStroke(False,0)
			
			# produce half-wave stroke on layer B
			self.addTriplet(c0,1)
			self.addTriplet(c1,1)
			self.addTriplet(c2,1)
			self.finishStroke(False,1)

			# start new  half-wave stroke on layer A
			# to be finished in next period
			self.addTriplet(c2,0)
			self.addTriplet(c3,0)
			
			# If on first period, store first point
			if period==0:
				self.firstPoint=c0
		else:
			self.addTriplet(c0)
			self.addTriplet(c1)
			self.addTriplet(c2)
			self.addTriplet(c3)
		
	def endOfStroke(self):
		closed=self.refStroke.isClosed()
		if not closed:
			p0=self.refStroke.getLastPoint()
			arcLen0=self.actualWaveLength*self.asymmetry/2
			arcLen1=self.actualWaveLength*(1-self.asymmetry)/2
			c0=self.ccpr.max(p0,arcLen1,arcLen0,self.amplitude,self.side)
			self.addTriplet(c0,0)
			self.finishStroke(False,0)
		else:
			if self.split:
				self.addTriplet(self.firstPoint,0)
				self.finishStroke(False,0)
			else:
				self.finishStroke(True,0)

# -----------------------------------------------------------------------------
# Sine phase sine wave renderer
# 
# Periods:
# 	Single mode:
# 		- Add the zero, extremum, zero, extremum
# 	Split mode:
# 		- Finish pending segment on layer A with zero 
# 		and first extremum
# 		- Add segment between two next extrema on layer B
# 		- Start new segment on layer A with second extremum
# 
# Start of stroke:
# 	Nothing to do
# 
# End of stroke:
# 	Open stroke:
# 		The process is conceptually different, but ends up being
# 		the same in both modes:
# 		
# 		Single mode:
# 			- Add last point on stroke (image of n*2*pi,0) to 
# 			list of points in layer A and finish stroke
# 		Split mode:
# 			- Add last point on stroke (image of n*2*pi,0) to
# 			segment on layer A and finish stroke
# 
# 	Closed stroke:
# 		Continous mode:
# 			- Nothing to do, just mark the result stroke as closed
# 			so this will generate the segment from the 2nd extremum
# 			of the last period to the first extremum of the first period.
# 		Split mode:
# 			- Finish pending segment on layer A with first extremum
# 			of first period.
# 
class SymmetricZeroWaveRenderer_Sin(WaveRenderer):
	
	# Curvy Control Points renderer
	ccpr=None
	
	def  __init__(self,ccpr,parameters,image,path,waveLength,amplitude,side,asymmetry,split):		
		self.ccpr=ccpr
		super(SymmetricZeroWaveRenderer_Sin, self).__init__(parameters,image,path,waveLength,amplitude,side,asymmetry,split)
		
	def startOfStroke(self):
		self.points=[[],[]]
		self.firstPoint=None

	def processPeriod(self,period,last):
		arcLen0=self.actualWaveLength*self.asymmetry/2
		arcLen1=self.actualWaveLength*(1-self.asymmetry)/2
		p0dist=period*self.actualWaveLength
		p1dist=p0dist+arcLen0
		p2dist=p1dist+arcLen1
		p3dist=p2dist+arcLen1
		p0=self.refStroke.getPointAtDist(p0dist)
		p1=self.refStroke.getPointAtDist(p1dist)
		p2=self.refStroke.getPointAtDist(p2dist)
		p3=self.refStroke.getPointAtDist(p3dist)
		
		c0=self.ccpr.zero(p0,arcLen0,arcLen0,self.amplitude,self.amplitude,self.side)
		c1=self.ccpr.max(p1,arcLen0,arcLen1,self.amplitude,self.side)
		c2=self.ccpr.zero(p2,arcLen1,arcLen1,self.amplitude,self.amplitude,-self.side)
		c3=self.ccpr.max(p3,arcLen1,arcLen0,self.amplitude,-self.side)

		if self.split:
			# finish previous half-wave on layer A
			self.addTriplet(c0,0)
			self.addTriplet(c1,0)
			self.finishStroke(False,0)
			
			# create half-wave on layer B
			self.addTriplet(c1,1)
			self.addTriplet(c2,1)
			self.addTriplet(c3,1)
			self.finishStroke(False,1)

			# start half-wave for layer B
			# to be finished in next period
			self.addTriplet(c3,0)
			
			# If on first period, store first point
			if period==0:
				self.firstPoint=c0
		else:
			self.addTriplet(c0)
			self.addTriplet(c1)
			self.addTriplet(c2)
			self.addTriplet(c3)

	def endOfStroke(self):
		closed=self.refStroke.isClosed()
		if not closed:
			p0=self.refStroke.getLastPoint()
			arcLen0=self.actualWaveLength*self.asymmetry/2
			c0=self.ccpr.zero(p0,arcLen0,arcLen0,self.amplitude,self.amplitude,self.side)
			self.addTriplet(c0,0)
			self.finishStroke(False,0)
		else:
			if self.split:
				self.addTriplet(self.firstPoint,0)
				self.finishStroke(False,0)
			else:
				self.finishStroke(True,0)



# -----------------------------------------------------------------------------
# Wave renderers for symmetrical extrema
# -----------------------------------------------------------------------------
class SymmetricMaxWaveRenderer_Cos(WaveRenderer):
	
	# Curvy Control Points renderer
	ccpr=None
	
	def  __init__(self,ccpr,parameters,image,path,waveLength,amplitude,side,asymmetry,split):		
		self.ccpr=ccpr
		super(SymmetricMaxWaveRenderer_Cos, self).__init__(parameters,image,path,waveLength,amplitude,side,asymmetry,split)
		
	def startOfStroke(self):
		self.points=[[],[]]
		self.firstPoint=None

	def processPeriod(self,period,last):
		arcLen0=self.actualWaveLength*self.asymmetry/2
		arcLen1=self.actualWaveLength*(1-self.asymmetry)/2
		p0dist=period*self.actualWaveLength
		p1dist=p0dist+arcLen0
		p2dist=p1dist+arcLen1
		p3dist=p2dist+arcLen1
		p0=self.refStroke.getPointAtDist(p0dist)
		p1=self.refStroke.getPointAtDist(p1dist)
		p2=self.refStroke.getPointAtDist(p2dist)
		p3=self.refStroke.getPointAtDist(p3dist)
		
		c0=self.ccpr.max(p0,arcLen0,arcLen0,self.amplitude,self.side)
		c1=self.ccpr.zero(p1,arcLen0,arcLen1,self.amplitude,self.amplitude,-self.side)
		c2=self.ccpr.max(p2,arcLen1,arcLen1,self.amplitude,-self.side)
		c3=self.ccpr.zero(p3,arcLen1,arcLen0,self.amplitude,self.amplitude,self.side)

		if self.split:
			if period:
				# finish half-wave stroke on layer A 
				self.addTriplet(c0,0)
				self.finishStroke(False,0)
			
			# produce half-wave stroke on layer B
			self.addTriplet(c0,1)
			self.addTriplet(c1,1)
			self.addTriplet(c2,1)
			self.finishStroke(False,1)

			# start new  half-wave stroke on layer A
			# to be finished in next period
			self.addTriplet(c2,0)
			self.addTriplet(c3,0)
			
			# If on first period, store first point
			if period==0:
				self.firstPoint=c0
		else:
			self.addTriplet(c0)
			self.addTriplet(c1)
			self.addTriplet(c2)
			self.addTriplet(c3)
		
	def endOfStroke(self):
		closed=self.refStroke.isClosed()
		if not closed:
			p0=self.refStroke.getLastPoint()
			arcLen0=self.actualWaveLength*self.asymmetry/2
			arcLen1=self.actualWaveLength*(1-self.asymmetry)/2
			c0=self.ccpr.max(p0,arcLen0,arcLen0,self.amplitude,self.side)
			self.addTriplet(c0,0)
			self.finishStroke(False,0)
		else:
			if self.split:
				self.addTriplet(self.firstPoint,0)
				self.finishStroke(False,0)
			else:
				self.finishStroke(True,0)

class SymmetricMaxWaveRenderer_Sin(WaveRenderer):
	
	# Curvy Control Points renderer
	ccpr=None
	
	def  __init__(self,ccpr,parameters,image,path,waveLength,amplitude,side,asymmetry,split):		
		self.ccpr=ccpr
		super(SymmetricMaxWaveRenderer_Sin, self).__init__(parameters,image,path,waveLength,amplitude,side,asymmetry,split)
		
	def startOfStroke(self):
		self.points=[[],[]]
		self.firstPoint=None

	def processPeriod(self,period,last):
		arcLen0=self.actualWaveLength*self.asymmetry/2
		arcLen1=self.actualWaveLength*(1-self.asymmetry)/2
		p0dist=period*self.actualWaveLength
		p1dist=p0dist+arcLen0
		p2dist=p1dist+arcLen0
		p3dist=p2dist+arcLen1
		p0=self.refStroke.getPointAtDist(p0dist)
		p1=self.refStroke.getPointAtDist(p1dist)
		p2=self.refStroke.getPointAtDist(p2dist)
		p3=self.refStroke.getPointAtDist(p3dist)
		
		c0=self.ccpr.zero(p0,arcLen1,arcLen0,self.amplitude,self.amplitude,self.side)
		c1=self.ccpr.max(p1,arcLen0,arcLen0,self.amplitude,self.side)
		c2=self.ccpr.zero(p2,arcLen0,arcLen1,self.amplitude,self.amplitude,-self.side)
		c3=self.ccpr.max(p3,arcLen1,arcLen1,self.amplitude,-self.side)

		if self.split:
			# finish previous half-wave on layer A
			self.addTriplet(c0,0)
			self.addTriplet(c1,0)
			self.finishStroke(False,0)
			
			# create half-wave on layer B
			self.addTriplet(c1,1)
			self.addTriplet(c2,1)
			self.addTriplet(c3,1)
			self.finishStroke(False,1)

			# start half-wave for layer B
			# to be finished in next period
			self.addTriplet(c3,0)
			
			# If on first period, store first point
			if period==0:
				self.firstPoint=c0
		else:
			self.addTriplet(c0)
			self.addTriplet(c1)
			self.addTriplet(c2)
			self.addTriplet(c3)

	def endOfStroke(self):
		closed=self.refStroke.isClosed()
		if not closed:
			p0=self.refStroke.getLastPoint()
			arcLen0=self.actualWaveLength*self.asymmetry/2
			arcLen1=self.actualWaveLength*(1-self.asymmetry)/2
			c0=self.ccpr.zero(p0,arcLen1,arcLen0,self.amplitude,self.amplitude,self.side)
			self.addTriplet(c0,0)
			self.finishStroke(False,0)
		else:
			if self.split:
				self.addTriplet(self.firstPoint,0)
				self.finishStroke(False,0)
			else:
				self.finishStroke(True,0)

# -----------------------------------------------------------------------------
# Startup functions
# -----------------------------------------------------------------------------

def isCosineType(phase):
	return phase % 2

def side(phase):
	return 1-2*(phase//2)

FUNCTYPE_SIN=0
FUNCTYPE_COS=1

PHASE_NAMES=['None','1/4','1/2','3/4']

def parametersString(shape,waveLength,amplitude,phase,asymmetry):
	return '%s(%d,%d,%s,%d)' % (shape,waveLength,amplitude,PHASE_NAMES[phase],asymmetry)


def sineWave(image,path,waveLength,amplitude,phase,asymmetry,split):
	parameters=parametersString('Sine',waveLength,amplitude,phase,asymmetry)
	if isCosineType(phase):
		SymmetricZeroWaveRenderer_Cos(SineControlPointsRenderer(),parameters,image,path,waveLength,amplitude,side(phase),asymmetry,split).run()
	else:
		SymmetricZeroWaveRenderer_Sin(SineControlPointsRenderer(),parameters,image,path,waveLength,amplitude,side(phase),asymmetry,split).run()
	
def ellipseWave(image,path,waveLength,amplitude,phase,asymmetry,split):
	parameters=parametersString('Ellipse',waveLength,amplitude,phase,asymmetry)
	if isCosineType(phase):
		SymmetricZeroWaveRenderer_Cos(EllipseControlPointsRenderer(),parameters,image,path,waveLength,amplitude,side(phase),asymmetry,split).run()
	else:
		SymmetricZeroWaveRenderer_Sin(EllipseControlPointsRenderer(),parameters,image,path,waveLength,amplitude,side(phase),asymmetry,split).run()
	
def circleWave(image,path,waveLength,amplitude,phase,asymmetry,split):
	parameters=parametersString('Circle',waveLength,amplitude,phase,asymmetry)
	if isCosineType(phase):
		SymmetricMaxWaveRenderer_Cos(CircleControlPointsRenderer(),parameters,image,path,waveLength,amplitude,side(phase),asymmetry,split).run()
	else:
		SymmetricMaxWaveRenderer_Sin(CircleControlPointsRenderer(),parameters,image,path,waveLength,amplitude,side(phase),asymmetry,split).run()
	
def squareWave(image,path,waveLength,amplitude,phase,asymmetry,split):
	parameters=parametersString('Square',waveLength,amplitude,phase,asymmetry)
	if isCosineType(phase):
		SquareWaveRenderer_Cos(parameters,image,path,waveLength,amplitude,side(phase),asymmetry,split).run()
	else:
		SquareWaveRenderer_Sin(parameters,image,path,waveLength,amplitude,side(phase),asymmetry,split).run()

def triangleWave(image,path,waveLength,amplitude,phase,asymmetry,split):
	parameters=parametersString('Triangle',waveLength,amplitude,phase,asymmetry)
	if isCosineType(phase):
		TriangleWaveRenderer_Cos(parameters,image,path,waveLength,amplitude,side(phase),asymmetry,split).run()
	else:
		TriangleWaveRenderer_Sin(parameters,image,path,waveLength,amplitude,side(phase),asymmetry,split).run()

# -----------------------------------------------------------------------------
# Registrations
# -----------------------------------------------------------------------------
AUTHOR='Ofnuts'
YEAR='2011'

whoiam='\n'+os.path.abspath(sys.argv[0])

REGISTRATION_IMAGE=(PF_IMAGE, 'image', 'Input image', None)
REGISTRATION_PATH=(PF_VECTORS, 'refpath', 'Input path', None)
REGISTRATION_WAVELENGTH=(PF_SPINNER, 'waveLength', 'Wavelength (pixels):', 50, (1, 1000, 1))
REGISTRATION_AMPLITUDE=(PF_SPINNER, 'amplitude', 'Amplitude (pixels):', 50, (1, 1000, 1))
REGISTRATION_PHASE=(PF_OPTION, 'phase', 'Wavelength shift:', 0, PHASE_NAMES)
REGISTRATION_ASYMMETRY_FULL=(PF_SPINNER, 'asymmetry', 'Asymmetry:', 0, (-50, +50, 1))
REGISTRATION_ASYMMETRY_LIMIT=(PF_SPINNER, 'asymmetry', 'Asymmetry:', 0, (-49, +49, 1))
REGISTRATION_SPLIT=(PF_TOGGLE, 'split', 'Split top and bottom:', 0)

MENU='<Vectors>/Decorate/Periodic wave'

register(
	'path-periodic-sine-wave',
	'Render sine wave along path...'+whoiam,
	'Render sine wave along path...',
	AUTHOR,
	AUTHOR,
	YEAR,
	'Sine...',
	'RGB*,GRAY*',
	[
		REGISTRATION_IMAGE,
		REGISTRATION_PATH,
		REGISTRATION_WAVELENGTH,
		REGISTRATION_AMPLITUDE,
		REGISTRATION_PHASE,
		REGISTRATION_ASYMMETRY_LIMIT,
		REGISTRATION_SPLIT
	],
	[],
	sineWave,
	menu=MENU,
)

register(
	'path-periodic-ellipse-wave',
	'Render ellipse wave along path...'+whoiam,
	'Render ellipse wave along path...',
	AUTHOR,
	AUTHOR,
	YEAR,
	'Ellipse...',
	'RGB*,GRAY*',
	[
		REGISTRATION_IMAGE,
		REGISTRATION_PATH,
		REGISTRATION_WAVELENGTH,
		REGISTRATION_AMPLITUDE,
		REGISTRATION_PHASE,
		REGISTRATION_ASYMMETRY_LIMIT,
		REGISTRATION_SPLIT
	],
	[],
	ellipseWave,
	menu=MENU,
)

register(
	'path-periodic-circle-wave',
	'Render circle wave along path...'+whoiam,
	'Render circle wave along path...',
	AUTHOR,
	AUTHOR,
	YEAR,
	'Circle...',
	'RGB*,GRAY*',
	[
		REGISTRATION_IMAGE,
		REGISTRATION_PATH,
		REGISTRATION_WAVELENGTH,
		REGISTRATION_AMPLITUDE,
		REGISTRATION_PHASE,
		REGISTRATION_ASYMMETRY_LIMIT,
		REGISTRATION_SPLIT
	],
	[],
	circleWave,
	menu=MENU,
)

register(
	'path-periodic-square-wave',
	'Render square wave along path...'+whoiam,
	'Render square wave along path...',
	AUTHOR,
	AUTHOR,
	YEAR,
	'Square...',
	'RGB*,GRAY*',
	[
		REGISTRATION_IMAGE,
		REGISTRATION_PATH,
		REGISTRATION_WAVELENGTH,
		REGISTRATION_AMPLITUDE,
		REGISTRATION_PHASE,
		REGISTRATION_ASYMMETRY_LIMIT,
		REGISTRATION_SPLIT
	],
	[],
	squareWave,
	menu=MENU,
)

register(
	'path-periodic-triangle-wave',
	'Render triangle wave along path...'+whoiam,
	'Render triangle wave along path...',
	AUTHOR,
	AUTHOR,
	YEAR,
	'Triangle...',
	'RGB*,GRAY*',
	[
		REGISTRATION_IMAGE,
		REGISTRATION_PATH,
		REGISTRATION_WAVELENGTH,
		REGISTRATION_AMPLITUDE,
		REGISTRATION_PHASE,
		REGISTRATION_ASYMMETRY_FULL,
		REGISTRATION_SPLIT
	],
	[],
	triangleWave,
	menu=MENU,
)

main()
print whoiam+': registration OK'